SystemOrganization addCategory: #'BootStrappedClasses-Monticello Support'!
SystemOrganization addCategory: #'BootStrappedClasses-Test'!

!ClassDescription methodsFor: '*BootStrappedClasses-accessing parallel hierarchy-override' stamp: 'Noury Bouraqadi 4/26/2006 19:14'!
isClassSide
	"Noury Bouraqadi 4/26/2006 19:08"
	"Avoids loops between base and meta-levels. As a consequence, a bootStrapped class answers false to  the isMeta message. This is good cause the the Smalltalk isMeta message actually tests whether the receiever is an implicit metaclass."
	
	self isBootStrapped ifTrue: [^false].
	^self == self classSide! !

!Behavior methodsFor: '*BootStrappedClasses' stamp: 'Noury Bouraqadi 4/27/2006 22:16'!
bootStrap
	"Noury Bouraqadi 4/14/2006 03:09"

	| allClassIVs allIVs |
	self isBootStrapped ifTrue: [^self].
	allClassIVs := self class allInstVarNames.
	allIVs := self allInstVarNames.
	allClassIVs = allIVs ifFalse: [
		^self error: 'Incompatible IVs between class and its metaclass'].
	self primitiveChangeClassTo: self basicNew.
! !

!Class methodsFor: '*BootStrappedClasses-monticello-override' stamp: 'Noury Bouraqadi 4/14/2006 14:01'!
asClassDefinition
	"Noury Bouraqadi 4/14/2006 14:01"
	
	^ self classDefinitionClass
		name: self name
		superclassName: self superclass name
		traitComposition: self traitComposition asString
		classTraitComposition: self class traitComposition asString
		category: self category 
		instVarNames: self instVarNames
		classVarNames: self classVarNames
		poolDictionaryNames: self poolDictionaryNames
		classInstVarNames: self class instVarNames
		type: self typeOfClass
		comment: self organization classComment	 asString
		commentStamp: self organization commentStamp	! !

!Class methodsFor: '*BootStrappedClasses-monticello' stamp: 'Noury Bouraqadi 4/26/2006 19:45'!
bootStrappedClassDefinitionClass
	"Noury Bouraqadi 4/14/2006 13:20"
	
	^BSCBootStrappedClassDefinition ! !

!Class methodsFor: '*BootStrappedClasses-monticello' stamp: 'Noury Bouraqadi 4/25/2006 22:32'!
classDefinitionClass
	"Noury Bouraqadi 3/23/2006 17:38"
	
	self isBootStrapped ifTrue: [^self bootStrappedClassDefinitionClass].
	^self plainClassDefinitionClass  ! !

!Class methodsFor: '*BootStrappedClasses-copying-override' stamp: 'Noury Bouraqadi 4/26/2006 19:39'!
copy
	"Noury Bouraqadi 4/26/2006 19:23"
	 
	| newClass |
	newClass := self metaclassCopy new
		superclass: superclass
		methodDict: self methodDict copy
		format: format
		name: name
		organization: self organization copy
		instVarNames: instanceVariables copy
		classPool: classPool copy
		sharedPools: sharedPools.
	Class instSize+1 to: self class instSize do:
		[:offset | newClass instVarAt: offset put: (self instVarAt: offset)].
	self isBootStrapped ifTrue: [newClass bootStrap].
	^ newClass! !

!Class methodsFor: '*BootStrappedClasses-copying' stamp: 'Noury Bouraqadi 4/26/2006 23:09'!
copyMetaclass
	"Noury Bouraqadi 4/26/2006 19:22"
	
	^self isBootStrapped not! !

!Class methodsFor: '*BootStrappedClasses-initialize-release' stamp: 'Noury Bouraqadi 4/26/2006 00:07'!
makeMetaclassObsolete
	"Noury Bouraqadi 4/26/2006 00:07"

	self isBootStrapped ifTrue: [^self "already obsolete"].	
	self class obsolete! !

!Class methodsFor: '*BootStrappedClasses-copying' stamp: 'Noury Bouraqadi 4/26/2006 19:39'!
metaclassCopy
	"Noury Bouraqadi 4/26/2006 19:22"
	
	self copyMetaclass ifFalse: [^self class].
	^self class copy! !

!Class methodsFor: '*BootStrappedClasses-intialize-release-override' stamp: 'Noury Bouraqadi 3/23/2006 07:19'!
obsolete
	"Noury Bouraqadi 3/23/2006 07:18"
	"Change the receiver and all of its subclasses to an obsolete class."
	
	self == Object 
		ifTrue: [^self error: 'Object is NOT obsolete'].
	self setName: 'AnObsolete' , self name.
	Object class instSize + 1 to: self class instSize do:
		[:i | self instVarAt: i put: nil]. "Store nil over class instVars."
	self classPool: nil.
	self sharedPools: nil.
	self makeMetaclassObsolete.
	super obsolete.! !

!Class methodsFor: '*BootStrappedClasses-monticello' stamp: 'Noury Bouraqadi 4/25/2006 22:32'!
plainClassDefinitionClass
	"Noury Bouraqadi 4/14/2006 14:00"
	
	^ MCClassDefinition! !

MCClassDefinition subclass: #BSCBootStrappedClassDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BootStrappedClasses-Monticello Support'!

!BSCBootStrappedClassDefinition commentStamp: 'Noury Bouraqadi 4/28/2006 00:46' prior: 0!
"Noury Bouraqadi 4/28/2006 00:46"

A BSCBootStrappedClassDefinition is a class definition for boot strapped. It supports saving/loading monticello packages with classes instances of them selves.
!

!BSCBootStrappedClassDefinition methodsFor: 'installing' stamp: 'Noury Bouraqadi 4/28/2006 00:45'!
load
	"Noury Bouraqadi 4/19/2006 16:42"
	
	| metaclass |
	super load.
	metaclass := self actualClass class.
	self actualClass bootStrap.
	self actualClass isBootStrapped ifTrue: [
		ChangeSet current removeClassChanges: metaclass name.
		metaclass obsolete]! !

!BSCBootStrappedClassDefinition methodsFor: 'installing' stamp: 'Noury Bouraqadi 4/19/2006 16:39'!
loadMetaclassOf: class
	"Noury Bouraqadi 4/19/2006 16:05"
	"Prepare bootStrapping"

	class class superclass: class superclass.
	super loadMetaclassOf: class	
! !

!BSCBootStrappedClassDefinition methodsFor: 'printing' stamp: 'Noury Bouraqadi 4/28/2006 01:49'!
printDefinitionOn: stream
	"Noury Bouraqadi 4/28/2006 01:44"

	self printStringClassIsBootStrappedOn: stream.	
	super printDefinitionOn: stream
! !

!BSCBootStrappedClassDefinition methodsFor: 'printing' stamp: 'Noury Bouraqadi 4/28/2006 01:50'!
printStringClassIsBootStrappedOn: stream
	"Noury Bouraqadi 4/28/2006 01:50"
	
	stream
		nextPutAll: self stringClassIsBootStrapped;
		cr; cr.
! !

!BSCBootStrappedClassDefinition methodsFor: 'printing' stamp: 'Noury Bouraqadi 4/28/2006 01:50'!
stringClassIsBootStrapped
	"Noury Bouraqadi 4/28/2006 01:46"
	
	^'"Boot strapped class"'! !

TestCase subclass: #BSCBootStrappedClassTest
	instanceVariableNames: 'class packageManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BootStrappedClasses-Test'!

!BSCBootStrappedClassTest methodsFor: 'setUp-tearDown' stamp: 'Noury Bouraqadi 4/27/2006 23:57'!
className
	"Noury Bouraqadi 4/27/2006 23:56"

	^#BSCDummy! !

!BSCBootStrappedClassTest methodsFor: 'accessing' stamp: 'Noury Bouraqadi 4/26/2006 23:26'!
packageName
	^#ClassesForBootStrappingTests! !

!BSCBootStrappedClassTest methodsFor: 'setUp-tearDown' stamp: 'Noury 12/20/2006 01:27'!
setUp
	"Noury Bouraqadi 6/26/2002 19:26"
	packageManager := MonticelloPackageManagerForTest forPackageNamed: self packageName.
	class := Class
				subclass: self className
				instanceVariableNames: 'x y'
				classVariableNames: ''
				poolDictionaries: ''
				category: packageManager category.
	class class instanceVariableNames: 'x y'.
	class bootStrap.
	class compile: 'x ^x';
		 compile: 'a: newA b: newB
	x := newB.
	y :=  newA';
		 compile: 'bar
	Transcript cr; show: ''Sub>>bar'''! !

!BSCBootStrappedClassTest methodsFor: 'setUp-tearDown' stamp: 'Noury Bouraqadi 4/27/2006 23:58'!
tearDown
	class isObsolete ifFalse: [
		class removeFromSystemUnlogged].
	ChangeSet current removeClassChanges: self className.
	packageManager cleanUp
! !

!BSCBootStrappedClassTest methodsFor: 'running' stamp: 'Noury Bouraqadi 4/27/2006 22:32'!
testChangeSet
	| changeSet |
	changeSet := ChangeSet current.
	self deny: (changeSet classChangeAt: class name) isEmpty! !

!BSCBootStrappedClassTest methodsFor: 'running' stamp: 'Noury Bouraqadi 4/26/2006 23:05'!
testCopy
	| newClass |
	newClass := class copy.
	self assert: newClass isBootStrapped! !

!BSCBootStrappedClassTest methodsFor: 'running' stamp: 'Noury Bouraqadi 4/26/2006 23:04'!
testIsMeta
	self deny: class isMeta! !

!BSCBootStrappedClassTest methodsFor: 'running' stamp: 'Noury 12/18/2006 23:23'!
testMonticello
	"Noury Bouraqadi 4/14/2006 03:37"
	| className changeSet implicitClassName |
	implicitClassName := class name , ' class'.
	changeSet := ChangeSet current.
	changeSet removeClassChanges: implicitClassName.
	packageManager savePackage.
	className := class name.
	packageManager unloadPackage.
	self
		shouldnt: [packageManager loadPackage]
		raise: Error.
	class := Smalltalk at: className.
	self assert: class isBootStrapped.
	self assert: (changeSet classChangeAt: implicitClassName) isEmpty.
	self deny: class isObsolete! !

BSCBootStrappedClassTest subclass: #BSCPackageDependencyTest
	instanceVariableNames: 'requiredPackageManager requiredClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BootStrappedClasses-Test'!

!BSCPackageDependencyTest methodsFor: 'accessing' stamp: 'Noury 12/18/2006 23:19'!
requiredClassName
	^ #ClassDefinedWithinTheRequiredToBeDeleted! !

!BSCPackageDependencyTest methodsFor: 'accessing' stamp: 'Noury 12/18/2006 23:03'!
requiredPackageName
	^#ClassesForPackageDependencyTestsToBeDeleted! !

!BSCPackageDependencyTest methodsFor: 'setUp-tearDown' stamp: 'Noury 12/20/2006 01:27'!
setUp
	super setUp.
	requiredPackageManager := MonticelloPackageManagerForTest forPackageNamed: self requiredPackageName.
	requiredPackageManager addRequiredPackageNamed: #BootStrappedClasses.
	packageManager addRequiredPackage: requiredPackageManager package.
	requiredClass := Object
				subclass: self requiredClassName
				instanceVariableNames: 'a b c'
				classVariableNames: ''
				poolDictionaries: ''
				category: requiredPackageManager category.
	requiredPackageManager savePackage! !

!BSCPackageDependencyTest methodsFor: 'setUp-tearDown' stamp: 'Noury 12/18/2006 23:04'!
tearDown
	super tearDown.
	requiredPackageManager cleanUp ! !

!BSCPackageDependencyTest methodsFor: 'running' stamp: 'Noury 12/18/2006 23:24'!
testLoading
	packageManager savePackage.
	packageManager unloadPackage.
	requiredPackageManager unloadPackage.
	self
		shouldnt: [packageManager loadPackage]
		raise: Error.
	Smalltalk
		at: self requiredClassName
		ifAbsent: [self fail]! !
